﻿/**
* Sample code for the GetCustomerInformation Canada Post service.
* 
* The GetCustomerInformation service is used to retrieve details on allowed
* payers, methods of payment and other information about a customer to determine
* the proper customer number for charging shipping costs.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetCustomerInformation
{
    class GetCustomerInformation
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Shipping_DotNet_Samples\SOAP\shipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            CustomerInfoPortTypeClient server = new CustomerInfoPortTypeClient("CustomerInfoPort", "https://ct.soa-gw.canadapost.ca/rs/soap/customerinfo");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getcustomerinformationrequest request = new getcustomerinformationrequest();
            request.locale = "FR";
            request.customernumber = mailedBy;

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getcustomerinformationresponse response = server.GetCustomerInformation(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getcustomerinformationresponseCustomer))
                {
                    getcustomerinformationresponseCustomer customer = (getcustomerinformationresponseCustomer)response.Item;
                    responseAsString += "Customer Number: " + customer.customernumber + "\r\n";
                    responseAsString += "\r\nContract Ids:\r\n";

                    foreach (String contractId in customer.contracts)
                    {
                        responseAsString += "- " + contractId + "\r\n";
                    }
                    responseAsString += "\r\nPayers:\r\n";
                    foreach (PayerType payer in customer.authorizedpayers)
                    {
                        responseAsString += "- Customer Number: " + payer.payernumber + "\r\n";
                        var i = 0;
                        foreach (String methodOfPayment in payer.methodsofpayment)
                        {
                            if (i == 0)
                            {
                                responseAsString += "  Payment Method:\r\n";
                            }
                            i++;
                            responseAsString += "  - " + methodOfPayment + "\r\n";
                        }
                        responseAsString += "\r\n";
                    }
                    if (customer.mobos != null)
                    {
                        foreach (var mobo in customer.mobos)
                        {
                            responseAsString += "Mobo: " + mobo + "\r\n";
                        }
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
