﻿/**
* Sample code for the GetShipmentReceipt Canada Post service.
* 
* The GetShipmentReceipt service is used to retrieve the shipment receipt 
* in XML format for a no-manifest shipment payed by Credit Card created by a prior "create shipment" call.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetShipmentReceipt
{
    class GetShipmentReceipt
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Shipping_DotNet_Samples\SOAP\shipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            ShipmentPortTypeClient server = new ShipmentPortTypeClient("ShipmentPort", "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getshipmentreceiptrequest request = new getshipmentreceiptrequest();
            request.locale = "EN";
            request.mailedby = mailedBy;
            request.shipmentid = "832091378834466920";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                server.Open();
                getshipmentreceiptresponse response = server.GetShipmentReceipt(request);
                // Retrieve values from response object
                if (response.Item.GetType() == typeof(ShipmentReceiptType))
                {
                    ShipmentReceiptType shipmentReceipt = (ShipmentReceiptType)response.Item;
                    if (shipmentReceipt.Item.GetType() == typeof(ShipmentReceiptTypeCcreceiptdetails))
                    {
                        ShipmentReceiptTypeCcreceiptdetails receiptDetails = (ShipmentReceiptTypeCcreceiptdetails)shipmentReceipt.Item;
                        responseAsString += "Auth Code: " + receiptDetails.authcode + "\r\n";
                        responseAsString += "Auth Timestamp: " + receiptDetails.authtimestamp + "\r\n";
                        responseAsString += "Charge Amount: " + receiptDetails.chargeamount + "\r\n";
                        responseAsString += "Currency: " + receiptDetails.currency + "\r\n";
                        responseAsString += "Card Type: " + receiptDetails.cardtype + "\r\n";
                        responseAsString += "Merchant Name: " + receiptDetails.merchantname + "\r\n";
                        responseAsString += "Merchant URL: " + receiptDetails.merchanturl + "\r\n";
                        responseAsString += "Name on Card: " + receiptDetails.nameoncard + "\r\n";
                        responseAsString += "Transaction Type: " + receiptDetails.transactiontype + "\r\n";
                    }
                    if (shipmentReceipt.Item.GetType() == typeof(ShipmentReceiptTypeCcreceiptdetails))
                    {
                        ShipmentReceiptTypeSupplieraccountreceiptdetails receiptDetails = (ShipmentReceiptTypeSupplieraccountreceiptdetails)shipmentReceipt.Item;
                        responseAsString += "Auth Code: " + receiptDetails.authcode + "\r\n";
                        responseAsString += "Auth Timestamp: " + receiptDetails.authtimestamp + "\r\n";
                        responseAsString += "Charge Amount: " + receiptDetails.chargeamount + "\r\n";
                        responseAsString += "Currency: " + receiptDetails.currency + "\r\n";
                        responseAsString += "Merchant Name: " + receiptDetails.merchantname + "\r\n";
                        responseAsString += "Merchant URL: " + receiptDetails.merchanturl + "\r\n";
                        responseAsString += "Supplier Id: " + receiptDetails.supplierid + "\r\n";
                        responseAsString += "Transaction Type: " + receiptDetails.transactiontype + "\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
                Console.WriteLine(faultEx);
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            server.Close();

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
