﻿/**
* Sample code for the TransmitShipments Canada Post service.
* 
* The TransmitShipments service is used to specify shipments to be included in a manifest. 
* Inclusion in a manifest is specified by group. Specific shipments may be excluded if 
* desired.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace TransmitShipments
{
    class TransmitShipments
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Shipping_DotNet_Samples\SOAP\shipping\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            ManifestPortTypeClient server = new ManifestPortTypeClient("ManifestPort", "https://ct.soa-gw.canadapost.ca/rs/soap/manifest/v8");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            transmitshipmentsrequest request = new transmitshipmentsrequest();
            request.locale = "FR";
            request.mailedby = mailedBy;

            // Create transmit object            
            ShipmentTransmitSetType transmit = new ShipmentTransmitSetType();
            request.transmitset = new ShipmentTransmitSetType();
            request.transmitset = transmit;
            transmit.groupids = new String[1];
            transmit.manifestaddress = new ManifestAddressType();
            transmit.manifestaddress.addressdetails = new AddressDetailsType();

            // Populate transmit object
            transmit.groupids[0] = "4326432";
            transmit.requestedshippingpoint = "K1K4T3";
            transmit.cpcpickupindicator = true;
            transmit.cpcpickupindicatorSpecified = true;
            transmit.detailedmanifests = true;
            transmit.methodofpayment = "Account";
            transmit.manifestaddress.manifestcompany = "MajorShop";
            transmit.manifestaddress.phonenumber = "514 829 5879";
            transmit.manifestaddress.addressdetails.addressline1 = "1230 Tako RD.";
            transmit.manifestaddress.addressdetails.city = "Ottawa";
            transmit.manifestaddress.addressdetails.provstate = "ON";
            transmit.manifestaddress.addressdetails.postalzipcode = "K2B8J6"; 

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                transmitshipmentsresponse response = server.TransmitShipments(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(ManifestsType))
                {
                    ManifestsType manifests = (ManifestsType)response.Item;
                    foreach (var manifestid in manifests.manifestid)
                    {
                        responseAsString += "Manifest Id:  " + manifestid + "\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
